@@ -1,3 +1,6 @@
+# docs/COPYING 2a + DRY: https://github.com/getmail6/getmail6
+# Please refer to the git history regarding who changed what and when in this file.
+
 '''Base and mix-in classes implementing retrievers (message sources getmail can
 retrieve mail from).
 
@@ -377,14 +380,18 @@ class POP3_SSL_EXTENDED(poplib.POP3_SSL):
             try:
                 self.sock = socket.socket(af, socktype, proto)
                 self.sock.connect(sa)
-            except socket.error as msg:
+            except socket.error as e:
+                msg = e
                 if self.sock:
                     self.sock.close()
                 self.sock = None
                 continue
             break
         if not self.sock:
-            raise socket.error(msg)
+            if isinstance(msg, Exception):
+              raise msg
+            else:
+              raise socket.error(msg)
         extra_args = { 'server_hostname': host }
         if self.ssl_version:
             extra_args['ssl_version'] = self.ssl_version
@@ -518,10 +525,14 @@ class IMAP4_SSL_EXTENDED(imaplib.IMAP4_SSL):
        self.ssl_ciphers = ssl_ciphers
        imaplib.IMAP4_SSL.__init__(self, host, port, keyfile, certfile)
 
-    def open(self, host='', port=imaplib.IMAP4_SSL_PORT):
+    def open(self, host='', port=imaplib.IMAP4_SSL_PORT, timeout=None):
        self.host = host
        self.port = port
        self.sock = socket.create_connection((host, port))
+
+       # Note timeout is available in python 3.9 in native imaplib's/ssl's open, but wrap_socket does not support it.  Keep it for the future.
+       self.timeout = timeout
+
        extra_args = { 'server_hostname': host }
        if self.ssl_version:
            extra_args['ssl_version'] = self.ssl_version
@@ -1560,8 +1571,8 @@ class IMAPRetrieverBase(RetrieverSkeleton):
             return {}
 
         ext = re.search(
-            b'X-GM-THRID (?P<THRID>\d+) X-GM-MSGID (?P<MSGID>\d+)'
-            b' X-GM-LABELS \((?P<LABELS>.*)\) UID',
+            rb'X-GM-THRID (?P<THRID>\d+) X-GM-MSGID (?P<MSGID>\d+)'
+            rb' X-GM-LABELS \((?P<LABELS>.*)\) UID',
             response[0]
         )
         if not ext:
