@@ -1,3 +1,6 @@
+# docs/COPYING 2a + DRY: https://github.com/getmail6/getmail6
+# Please refer to the git history regarding who changed what and when in this file.
+
 '''Utility classes and functions for getmail.
 '''
 
@@ -69,6 +72,11 @@ try:
         gnomekeyring = None
 except ImportError:
     gnomekeyring = None
+# Optional Python keyring integration
+try:
+    import keyring
+except ImportError:
+    keyring = None
 
 from getmailcore.exceptions import *
 
@@ -339,6 +347,9 @@ def deliver_maildir(maildirpath, data, hostname, dcount=None, filemode=0o600):
 
     # Move message file from Maildir/tmp to Maildir/new
     try:
+        # #https://pypi.org/project/getmail_shutils/#description
+        # #Version of getmail using shutil instead of os.link to allow it to be used with a shared destination folder in a VM.
+        # shutil.copyfile(fname_tmp, fname_new)
         os.link(fname_tmp, fname_new)
         os.unlink(fname_tmp)
 
@@ -597,7 +608,10 @@ def check_ssl_ciphers(conf):
 #######################################
 keychain_password = None
 if os.name == 'posix':
-    if os.path.isfile(osx_keychain_binary):
+    if keyring:
+        def keychain_password(user, server, protocol, logger):
+            return keyring.get_password(server,user)
+    elif os.path.isfile(osx_keychain_binary):
         def keychain_password(user, server, protocol, logger):
             """Mac OSX: return a keychain password, if it exists.  Otherwise, return
 
@@ -634,7 +648,7 @@ if os.name == 'posix':
                         pw = pw[1:-1]
                     password = pw
             if password is None:
-                logger.debug('No keychain password found for %s %s %s'
+                logger.debug('No keychain password found for %s %s %s\n'
                              % (user, server, protocol))
             return password
     elif gnomekeyring:
@@ -655,7 +669,7 @@ if os.name == 'posix':
 
                 #logger.trace('got keyring result %s' % str(secret))
             except gnomekeyring.NoMatchError:
-                logger.debug('gnome-keyring does not know password for %s %s %s'
+                logger.debug('gnome-keyring does not know password for %s %s %s\n'
                              % (user, server, protocol))
                 return None
 
@@ -682,7 +696,7 @@ def get_password(label, user, server, protocol, logger):
     # try keychain/keyrings first, where available
     password = keychain_password(user, server, protocol, logger)
     if password:
-        logger.debug('using password from keychain/keyring')
+        logger.debug('using password from keychain/keyring\n')
     else:
         # no password found (or not on OSX), prompt in the usual way
         password = getpass.getpass('Enter password for %s:  ' % label)
